#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
using TradingStudies.NinjaScript.Utility;
using System.IO;
using System.Collections;
using System.Windows.Forms;
#endregion

namespace NinjaTrader.Strategy
{
    [Description("Scarlet Willow - A JMF3 Inception")]
    public class _ScarletWillowLiveTrader : Strategy
    {
        #region Trader Variables
        //DEFAULT SETTINGS FOR NQ 5 RANGE
		//Time Filter
        private int hourStart = 6; 		//My trading time; have not optimized for full day trading
        private int minuteStart = 30; 
        private int hourEnd = 8; 
        private int minuteEnd = 0;
		//P&L Filter
        private int maxWin = 1000;				
        private int maxLoss = 300; 			
		private int priorTradesCount = 0;
		private double priorTradesCumProfit = 0;
		//ATR Filter
		private bool bigATRFilter = false;
		private int bigATRPeriod = 107;			
		private double bigATRMultiplier = 5;  
		private int bigATRSmoothing = 21;		//use inhibits trading, but greatly increases in probabily; Multiplier = 6, Period = 9 (89% win rate Q1 2013)
		//ATR Trigger
		private int aTRPeriod = 14;          
		private double aTRMultiplier = 4.58; 	
		private int aTRSmoothing = 15;       	
		//CCIFilter
        private int cCIFilterPeriod = 71; 		
        private int cCIFilterLevel = 0; 
		private bool useCCIFilter = true;
		private bool useCCIOBOS = true;
		//Targets & Stops 
		private int target1=7;  				
		private int target2=34;					
		private int stop1=12;					//14 maximizes win rate, 10 or 12 maximizes profit, 9 also works
		private int quantity = 1;
		private int trailTicks = 1;				
		//Keltner Settings
		private int keltnerPeriod = 107;			
		private double keltnerMultiplier = 9.5;
		private bool useKeltnerExit = true;
		private bool useMAFilter = true;
		private int distanceToMA = 5;
		private bool midbandBias = true;
		//MA Settings
		private bool useMARisingFalling = true;
		private int mAPeriod = 67;
		//3BR
		private bool threeBRExit; 
		//Direction filter
		private bool longs = true;
		private bool shorts = true;
		//Re-Entry Trade
		private int reEntryTickDistance = 0;
		private bool useCCIReEntry = true;
		private bool use2BarEntry = true;
		//3BR Trade Entries - countertrend is best 
		private bool useMABiasFilter = false;
		private bool useDistanceToMAFilter = false;
		private bool useKeltnerFilter = true;		
		private int distanceToMA3BR = 5;
		private int distanceToKeltner = 12;			//1 maximizes win rate, but fairly infrequent
		private int target2BR = 17;
		//Box Trade
		private int boxTradeTarget1 = 3;
		private int boxTradeTarget2 = 6;
		private int boxTradeStop = 12;
		//Pullback Pattern
		private bool pullbackPattern = true;
        #endregion
		
		#region Button Variables
		private string trade = String.Empty;
		//Button Vars
		private System.Windows.Forms.ToolStrip strip = null;
		private System.Windows.Forms.Control[] controls    = null;

		private System.Windows.Forms.ToolStripButton btnAuto = null;
		private System.Windows.Forms.ToolStripButton btnLongEntry = null;
		private System.Windows.Forms.ToolStripButton btnShortEntry = null;
		private System.Windows.Forms.ToolStripButton btnBasicTrend = null;
		private System.Windows.Forms.ToolStripButton btnReEntry = null;
		private System.Windows.Forms.ToolStripButton btn3BR = null;
		private System.Windows.Forms.ToolStripButton btnBoxBreak = null;
		private System.Windows.Forms.ToolStripButton btnManualLongEntry = null;
		private System.Windows.Forms.ToolStripButton btnManualShortEntry = null;
		private System.Windows.Forms.ToolStripButton btnOrderType1 = null;
		private System.Windows.Forms.ToolStripButton btnClose = null;
		private System.Windows.Forms.ToolStripSeparator sepL = null;
		private System.Windows.Forms.ToolStripSeparator sep1 = null;
		private System.Windows.Forms.ToolStripSeparator sep2 = null;
		private System.Windows.Forms.ToolStripSeparator sep3 = null;
		private System.Windows.Forms.ToolStripSeparator sep4 = null;
		private System.Windows.Forms.ToolStripSeparator sep5 = null;
		private System.Windows.Forms.ToolStripSeparator sep6 = null;
		private System.Windows.Forms.ToolStripSeparator sep7 = null;
		private System.Windows.Forms.ToolStripSeparator sep8 = null;
		private System.Windows.Forms.ToolStripSeparator sep9 = null;
		private System.Windows.Forms.ToolStripSeparator sep10 = null;
		private System.Windows.Forms.ToolStripSeparator sepR = null;
		
		private Font boldFont = new Font("Arial", 8,FontStyle.Bold);
		private Font regularFont = new Font("Arial", 8);
		private bool buttonsloaded = false;
		private string manualTemplate = "NQATM";
		private string boxTemplate = "NQBox";
		private int barsCancel = 5;
		
		private string	atmStrategyId		= string.Empty;
		private string	orderId				= string.Empty;
		
		int currentBar = 0;
		#endregion
		
        protected override void Initialize()
        {
	
			#region Add Indicators, Set Targets, Entries Per Direction
            Add(TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR));
			if (bigATRFilter == true)
				{Add(TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR));}
			//Longs Targets
            SetProfitTarget("LongBT1", CalculationMode.Ticks, target1);
			SetProfitTarget("LongBT2", CalculationMode.Ticks, target2);
			SetProfitTarget("RELong1", CalculationMode.Ticks, target1);
			SetProfitTarget("RELong2", CalculationMode.Ticks, target2);
			//Shorts Targets
            SetProfitTarget("ShortBT1", CalculationMode.Ticks, target1);
			SetProfitTarget("ShortBT2", CalculationMode.Ticks, target2);
			SetProfitTarget("REShort1", CalculationMode.Ticks, target1);
			SetProfitTarget("REShort2", CalculationMode.Ticks, target2);
			//3BR
			SetProfitTarget("3BRLong1",CalculationMode.Ticks,target1);
			SetProfitTarget("3BRLong2",CalculationMode.Ticks,target2BR);
			SetProfitTarget("3BRShort1",CalculationMode.Ticks,target1);
			SetProfitTarget("3BRShort2",CalculationMode.Ticks,target2BR);
			//Box Breakout
			SetProfitTarget("BoxLong1",CalculationMode.Ticks,boxTradeTarget1);
			SetProfitTarget("BoxLong2",CalculationMode.Ticks,boxTradeTarget2);
			SetProfitTarget("BoxShort1",CalculationMode.Ticks,boxTradeTarget1);
			SetProfitTarget("BoxShort2",CalculationMode.Ticks,boxTradeTarget2);
			//Dual Entries
            CalculateOnBarClose = true;
			EntriesPerDirection = 2; 
   			EntryHandling = EntryHandling.AllEntries;
			#endregion	
        }
		protected override void OnStartUp()
		{
			#region Set Up Buttons
			System.Windows.Forms.Control[] controls = ChartControl.Controls.Find("tsrTool", false);

			if (controls.Length > 0) {
				
				ToolStripButton btnTemp = new System.Windows.Forms.ToolStripButton("temp");
				btnTemp = null;

				strip = (System.Windows.Forms.ToolStrip)controls[0];

				sepL = new ToolStripSeparator();
				strip.Items.Add(sepL);
				
				btnAuto = new System.Windows.Forms.ToolStripButton("btnAuto");
				btnAuto.Font = regularFont ;    
				btnAuto.Font = boldFont ;
				btnAuto.ForeColor = Color.White;
				btnAuto.BackColor= Color.Green;    
				btnAuto.Text = "Auto On";
				btnAuto.ToolTipText = "Turn auto trading on or off";

				strip.Items.Add(btnAuto);
				btnAuto.Click += btnAuto_Click;
				
				sep1 = new ToolStripSeparator();
				strip.Items.Add(sep1);

				btnLongEntry = new System.Windows.Forms.ToolStripButton("btnLongEntry");
				btnLongEntry.Font = boldFont ;
				btnLongEntry.ForeColor = Color.Green;
				btnLongEntry.BackColor=Color.Transparent;
				btnLongEntry.Text = "Longs On";
				btnLongEntry.ToolTipText = "Permits Long Entries";

				strip.Items.Add(btnLongEntry);
				btnLongEntry.Click += btnLongEntry_Click;
				
				sep2 = new ToolStripSeparator();
				strip.Items.Add(sep2);

				btnShortEntry = new System.Windows.Forms.ToolStripButton("btnShortEntry");
				btnShortEntry.Font = boldFont ;
				btnShortEntry.ForeColor = Color.Green;
				btnShortEntry.BackColor=Color.Transparent;
				btnShortEntry.Text = "Shorts On";
				btnShortEntry.ToolTipText = "Permits Short Entries";

				strip.Items.Add(btnShortEntry);
				btnShortEntry.Click += btnShortEntry_Click;
				
				sep3 = new ToolStripSeparator();
				strip.Items.Add(sep3);
				
				btnBasicTrend = new System.Windows.Forms.ToolStripButton("btnBasicTrend");
				btnBasicTrend.Font = boldFont ;
				btnBasicTrend.ForeColor = Color.Green;
				btnBasicTrend.BackColor=Color.Transparent;
				btnBasicTrend.Text = "BasicTrend On";
				btnBasicTrend.ToolTipText = "Turns on and off Basic Trend Entries";

				strip.Items.Add(btnBasicTrend);
				btnBasicTrend.Click += btnBasicTrend_Click;
				
				sep4 = new ToolStripSeparator();
				strip.Items.Add(sep4);
				
				btnReEntry = new System.Windows.Forms.ToolStripButton("btnReEntry");
				btnReEntry.Font = boldFont ;
				btnReEntry.ForeColor = Color.Green;
				btnReEntry.BackColor=Color.Transparent;
				btnReEntry.Text = "ReEntry On";
				btnReEntry.ToolTipText = "Turns on and off Trend Re-Entries";

				strip.Items.Add(btnReEntry);
				btnReEntry.Click += btnReEntry_Click;
				
				sep5 = new ToolStripSeparator();
				strip.Items.Add(sep5);
				
				btn3BR = new System.Windows.Forms.ToolStripButton("btn3BR");
				btn3BR.Font = boldFont ;
				btn3BR.ForeColor = Color.Green;
				btn3BR.BackColor=Color.Transparent;
				btn3BR.Text = "3BR On";
				btn3BR.ToolTipText = "Turns on and off 3BR (Three Bar Reversal) Trades";

				strip.Items.Add(btn3BR);
				btn3BR.Click += btn3BR_Click;
				
				sep6 = new ToolStripSeparator();
				strip.Items.Add(sep6);
				
				btnBoxBreak = new System.Windows.Forms.ToolStripButton("btnBoxBreak");
				btnBoxBreak.Font = boldFont ;
				btnBoxBreak.ForeColor = Color.Green;
				btnBoxBreak.BackColor=Color.Transparent;
				btnBoxBreak.Text = "BoxBreak On";
				btnBoxBreak.ToolTipText = "Turns on and off Box Breakout Trades";

				strip.Items.Add(btnBoxBreak);
				btnBoxBreak.Click += btnBoxBreak_Click;
				
				sep7 = new ToolStripSeparator();
				strip.Items.Add(sep7);
				
				btnManualLongEntry = new System.Windows.Forms.ToolStripButton("btnManualLongEntry");
				btnManualLongEntry.Font = boldFont ;
				btnManualLongEntry.ForeColor = Color.White;
				btnManualLongEntry.BackColor=Color.Green;
				btnManualLongEntry.Text = "Manual Long";
				btnManualLongEntry.ToolTipText = "Submit a buy stop market order at the projected range bar close or a long market order at current price";

				strip.Items.Add(btnManualLongEntry);
				btnManualLongEntry.Click += btnManualLongEntry_Click;
				
				sep8 = new ToolStripSeparator();
				strip.Items.Add(sep8);

				btnManualShortEntry = new System.Windows.Forms.ToolStripButton("btnManualShortEntry");
				btnManualShortEntry.Font = boldFont ;
				btnManualShortEntry.ForeColor = Color.White;
				btnManualShortEntry.BackColor=Color.Red;
				btnManualShortEntry.Text = "Manual Short";
				btnManualShortEntry.ToolTipText = "Submit a sell stop market order at the projected range bar close or a short market order at current price";

				strip.Items.Add(btnManualShortEntry);
				btnManualShortEntry.Click += btnManualShortEntry_Click;
				
				sep9 = new ToolStripSeparator();
				strip.Items.Add(sep9);
				
				btnOrderType1 = new System.Windows.Forms.ToolStripButton("btnOrderType1");
				btnOrderType1.Font = boldFont ;
				btnOrderType1.ForeColor = Color.Green;
				btnOrderType1.BackColor=Color.Transparent;
				btnOrderType1.Text = "Stop Market";
				btnOrderType1.ToolTipText = "Select between stop market orders and market orders";
				
				strip.Items.Add(btnOrderType1);
				btnOrderType1.Click += btnOrderType1_Click;
				
				sep10 = new ToolStripSeparator();
				strip.Items.Add(sep10);
				
				btnClose = new System.Windows.Forms.ToolStripButton("btnClose");
				btnClose.Font = boldFont ;
				btnClose.ForeColor = Color.Gainsboro;
				btnClose.BackColor=Color.DarkGoldenrod;
				btnClose.Text = "Close";
				btnClose.ToolTipText = "Close Any Open Position";

				strip.Items.Add(btnClose);
				btnClose.Click += btnClose_Click;
				
				sepR = new ToolStripSeparator();
				strip.Items.Add(sepR);
				

				buttonsloaded=true;	
			}
			#endregion
		}
		
        protected override void OnBarUpdate()
        {
			#region Performance Filter
			if (Bars.FirstBarOfSession)
			{// Store the strategy's prior cumulated realized profit and number of trades
				priorTradesCount = Performance.RealtimeTrades.Count;
				priorTradesCumProfit = Performance.RealtimeTrades.TradesPerformance.Currency.CumProfit;
			}
			// Stop trading if Win max or loss max is reached.
			if (Performance.RealtimeTrades.TradesPerformance.Currency.CumProfit - priorTradesCumProfit >= maxWin
				|| Performance.RealtimeTrades.TradesPerformance.Currency.CumProfit - priorTradesCumProfit <= -maxLoss)
				{return;}
				#endregion
				
			#region Position & Profit Display
			double dailyTradeCount = Performance.AllTrades.Count - priorTradesCount;
			double dailyPL = Performance.AllTrades.TradesPerformance.Currency.CumProfit - priorTradesCumProfit;
			DrawTextFixed("Display","AUTOBoT STATS"+"\n"+"Market Position is Currently: "+Position.MarketPosition.ToString()+"\n"+"@ Price: "+Position.AvgPrice +"\n"+"\n"+"Daily P&L: "+dailyPL.ToString()+
				"\n"+"Trades Taken: "+dailyTradeCount.ToString(),TextPosition.TopLeft,Color.Black,new Font("Arial", 10,FontStyle.Regular),Color.Transparent,Color.Blue,2);
			#endregion
				
			if(Historical)
				return;
			
			#region Special Exit Conditions	
			if (Position.MarketPosition == MarketPosition.Long)
				{
					if (useKeltnerExit == true && High[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Upper[0])
						{AtmStrategyClose(atmStrategyId);}
					if (High[1]>High[2]&&High[2]>High[3] && High[3]>High[4] && Low[1]>Low[2] && Low[2]>Low[3] && Low[3]>Low[4]
						&& Close[0]<Open[0]	&& Close[1]>Open[1] && Close[2]>Open[2] && Close[3]>Open[3]
						&& threeBRExit == true)
						{AtmStrategyClose(atmStrategyId);}
					if (useCCIOBOS == true && CCI(cCIFilterPeriod)[0]>=200)
						{AtmStrategyClose(atmStrategyId);}
				}	
			// Set exit procedures if in a short position
			else if (Position.MarketPosition == MarketPosition.Short)
				{
					if (useKeltnerExit == true && Low[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Lower[0])
						{AtmStrategyClose(atmStrategyId);}
					if (Low[1]<Low[2]&&Low[2]<Low[3]&&Low[3]<Low[4] && High[1]<High[2] && High[2]<High[3]
						&& High[3]<High[4]&&Close[0]>Open[0] && Close[1]<Open[1] && Close[2]<Open[2] && Close[3]<Open[3] 
						&& threeBRExit == true)
						{AtmStrategyClose(atmStrategyId);}
					if (useCCIOBOS == true && CCI(cCIFilterPeriod)[0]<=-200)
						{AtmStrategyClose(atmStrategyId);}
				}
			#endregion	
				
			#region Basic Trend Trades
			{
			if (CrossAbove(Close, TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend, 1)
				&& ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& btnLongEntry.Text == "Longs On"
				&& btnAuto.Text == "Auto On"
				&& btnBasicTrend.Text == "BasicTrend On"
				&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
				&& (useMAFilter == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] - distanceToMA*TickSize || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]+2*TickSize)
				&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& (bigATRFilter == false || Close[0]> TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, 
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0])
				&& (useMARisingFalling == false || Rising(WMA(mAPeriod))))
				{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Buy,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					Print("Basic Trend Trade Taken");
				}
			// Short entry trigger
			if (CrossBelow(Close, TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend, 1)
				&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] <= -CCIFilterLevel)
				&& ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& btnShortEntry.Text == "Shorts On"
				&& btnAuto.Text == "Auto On"
				&& btnBasicTrend.Text == "BasicTrend On"
				&& (useMAFilter == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + distanceToMA*TickSize || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + 2*TickSize)
				&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& (bigATRFilter == false || Close[0]<TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA,
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0])
				&& (useMARisingFalling == false || Falling(WMA(mAPeriod))))
						{
							if (orderId.Length > 0)
								{
									string[] status = GetAtmStrategyEntryOrderStatus(orderId);
									if (status.GetLength(0) > 0)
										{
											if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
												orderId = string.Empty;
										}
									
									else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
										atmStrategyId = string.Empty;    
								}
							atmStrategyId = GetAtmStrategyUniqueId();
							orderId = GetAtmStrategyUniqueId();
							AtmStrategyCreate(OrderAction.Sell,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
						}
			}
			#endregion
			
			#region Re-Entry Trend Trade
			//Long Re-Entry
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& btnAuto.Text == "Auto On"
				&& btnLongEntry.Text == "Longs On"
				&& btnReEntry.Text == "ReEntry On"
				&& (use2BarEntry == true || Close[0]>Open[0] && Low[0]<=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0]+reEntryTickDistance*TickSize
				&& High[0]>TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0]+reEntryTickDistance*TickSize
				&& Close[0]>TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0])
				&& (use2BarEntry == false || Close[1]>=Open[2] && Close[0]>=Open[0] && Low[1]<=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[1]+reEntryTickDistance*TickSize
				&& High[1]>TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[1]+reEntryTickDistance*TickSize
				&& Close[1]>TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[1])
				&& (bigATRFilter == false || Close[0]> TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, 
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).UpTrend[0])
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
				&& (useMAFilter == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] - distanceToMA*TickSize || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]+2*TickSize)
				&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]))
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Buy,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& btnAuto.Text == "Auto On"
				&& btnLongEntry.Text == "Longs On"
				&& btnReEntry.Text == "ReEntry On"
				&& pullbackPattern == true
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
				&& (useMAFilter == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] - distanceToMA*TickSize || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]+2*TickSize)
				&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
				&& (Close[0]>Open[0] && Close[1]<Open[1] && Close[2]<Open[2] && Close[3]<Open[3] && Close[4]<Open[4] && Low[0]>=Low[3]
				|| Close[0]>Open[0] && Close[1]<Open[1] && Close[2]>Open[2] && Close[3]<Open[3] && Close[4]<Open[4] && Low[0]<Low[1] && Low[1]<Low[2] && Low[2]<Low[3]))
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Buy,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					}
				
			//Short Re-Entry
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& btnAuto.Text == "Auto On"
				&& btnShortEntry.Text == "Shorts On"
				&& btnReEntry.Text == "ReEntry On"
				&& (use2BarEntry == true || Close[0]<Open[0] && High[0]>=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0]-reEntryTickDistance*TickSize
				&& Low[0]<TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0]-reEntryTickDistance*TickSize
				&& Close[0]<TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0])
				&& (use2BarEntry == false || Close[1]<Open[1] && Close[0]<Open[0] && High[1]>=TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[1]-reEntryTickDistance*TickSize
				&& Low[1]<TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[1]-reEntryTickDistance*TickSize
				&& Close[1]<TSSuperTrend(aTRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA, aTRMultiplier, aTRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[1])
				&& (bigATRFilter == false || Close[0]<TSSuperTrend(bigATRPeriod, TradingStudies.NinjaScript.Utility.MovingAverageType.HMA,
					bigATRMultiplier, bigATRSmoothing, TradingStudies.NinjaScript.Utility.SuperTrendMode.ATR).DownTrend[0])
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
				&& (useMAFilter == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + distanceToMA*TickSize || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + 2*TickSize)
				&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]))
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Sell,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					}
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
				&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
				&& btnAuto.Text == "Auto On"
				&& btnShortEntry.Text == "Shorts On"
				&& btnReEntry.Text == "ReEntry On"
				&& shorts == true
				&& pullbackPattern == true
				&& (useCCIReEntry == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
				&& (useMAFilter == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + distanceToMA*TickSize || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0] + 2*TickSize)
				&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0]))
				{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Sell,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					}
			#endregion
					
			#region 3BR Trade
			//3BR Long
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& Low[1]<Low[2]&&Low[2]<Low[3]&&Low[3]<Low[4] && High[1]<High[2] && High[2]<High[3]
						&& High[3]<High[4]&&Close[0]>Open[0] && Close[1]<Open[1] && Close[2]<Open[2] && Close[3]<Open[3]
						&& (useMABiasFilter == false || Low[1]>= SMA(keltnerPeriod)[1])
						&& (useDistanceToMAFilter == false || Low[1]<= WMA(mAPeriod)[1]+distanceToMA3BR*TickSize)
						&& (useKeltnerFilter == false || Low[1]<=KeltnerChannel(keltnerMultiplier,keltnerPeriod).Lower[0]+distanceToKeltner*TickSize)
						&& btn3BR.Text == "3BR On"
						&& btnAuto.Text == "Auto On"
						&& btnLongEntry.Text == "Longs On")
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Buy,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					}
			//3BR Short
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[1]>High[2]&&High[2]>High[3] && High[3]>High[4] && Low[1]>Low[2] && Low[2]>Low[3] && Low[3]>Low[4]
						&& Close[0]<Open[0]	&& Close[1]>Open[1] && Close[2]>Open[2] && Close[3]>Open[3]
						&& (useMABiasFilter == false || High[1]<= SMA(keltnerPeriod)[1])
						&& (useDistanceToMAFilter == false || High[1]>= WMA(mAPeriod)[1]-distanceToMA3BR*TickSize)
						&& (useKeltnerFilter == false || High[1]>=KeltnerChannel(keltnerMultiplier,keltnerPeriod).Upper[0]-distanceToKeltner*TickSize)
						&& btn3BR.Text == "3BR On"
						&& btnAuto.Text == "Auto On"
						&& btnShortEntry.Text == "Shorts On")
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Sell,OrderType.Market,0,0,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
					}
				#endregion		
				
			#region Box Breakout Trade
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[0]<High[3]+2*TickSize && High[0]<High[2]+2*TickSize && High[0]<High[1]+2*TickSize && Low[0]>Low[3]-2*TickSize
						&& Low[0]>Low[2]-2*TickSize && Low[0]>Low[1]-2*TickSize
						&& btnBoxBreak.Text == "BoxBreak On"
						&& btnAuto.Text == "Auto On"
						&& btnLongEntry.Text == "Longs On"
						&& (midbandBias == false || Close[0]>KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] > CCIFilterLevel)
						&& (useMARisingFalling == false || Rising(WMA(mAPeriod))))
				{
					double range = MAX(Highs[0],3)[0]-MIN(Lows[0],3)[0];
					double orderLocation = MAX(Highs[0],3)[0] + range/2;
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Buy,OrderType.Stop,0,orderLocation,TimeInForce.Gtc,orderId,boxTemplate,atmStrategyId);
					currentBar = CurrentBar;
					}
					if (orderId.Length > 0 && CurrentBar - currentBar >= barsCancel)
					{
						AtmStrategyCancelEntryOrder(atmStrategyId);
					}
				}
					//box shorts
			if (ToTime(Time[0]) >= ToTime(hourStart, minuteStart, 0)
						&& ToTime(Time[0]) <= ToTime(hourEnd, minuteEnd, 0)
						&& High[0]<High[3]+2*TickSize && High[0]<High[2]+2*TickSize && High[0]<High[1]+2*TickSize && Low[0]>Low[3]-2*TickSize
						&& Low[0]>Low[2]-2*TickSize && Low[0]>Low[1]-2*TickSize
				 		&& btnAuto.Text == "Auto On"
						&& btnShortEntry.Text == "Shorts On"
						&& btnBoxBreak.Text == "BoxBreak On"
						&& (midbandBias == false || Close[0]<KeltnerChannel(keltnerMultiplier,keltnerPeriod).Midline[0])
						&& (useCCIFilter == false || CCI(CCIFilterPeriod)[0] < CCIFilterLevel)
						&& (useMARisingFalling == false || Falling(WMA(mAPeriod))))
				{
					double range = MAX(Highs[0],3)[0]-MIN(Lows[0],3)[0];
					double orderLocationShort = MIN(Lows[0],3)[0] - range/2;
					{
					if (orderId.Length > 0)
						{
							string[] status = GetAtmStrategyEntryOrderStatus(orderId);
							if (status.GetLength(0) > 0)
								{
									if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
										orderId = string.Empty;
								}	
							else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
								atmStrategyId = string.Empty;    
						}
					atmStrategyId = GetAtmStrategyUniqueId();
					orderId = GetAtmStrategyUniqueId();
					AtmStrategyCreate(OrderAction.Sell,OrderType.Stop,0,orderLocationShort,TimeInForce.Gtc,orderId,boxTemplate,atmStrategyId);
					currentBar = CurrentBar;
					}
					if(orderId.Length > 0 && CurrentBar - currentBar >= barsCancel)
					{
						AtmStrategyCancelEntryOrder(atmStrategyId);
					}
				}
			#endregion		
		}
	#region Buttons Dispose	
	public override void Dispose()
	{
	if (buttonsloaded==true)
	{
	strip.Items.Remove(btnAuto);
	strip.Items.Remove(btnLongEntry);
	strip.Items.Remove(btnShortEntry);
	strip.Items.Remove(btnBasicTrend);
	strip.Items.Remove(btnReEntry);
	strip.Items.Remove(btn3BR);
	strip.Items.Remove(btnBoxBreak);
	strip.Items.Remove(btnManualLongEntry);
	strip.Items.Remove(btnManualShortEntry);
	strip.Items.Remove(btnClose);
	strip.Items.Remove(btnOrderType1);
	strip.Items.Remove(sepL);
	strip.Items.Remove(sep1);
	strip.Items.Remove(sep2);
	strip.Items.Remove(sep3);
	strip.Items.Remove(sep4);
	strip.Items.Remove(sep5);
	strip.Items.Remove(sep6);
	strip.Items.Remove(sep7);
	strip.Items.Remove(sep8);
	strip.Items.Remove(sep9);
	strip.Items.Remove(sep10);
	strip.Items.Remove(sepR);
	RemoveDrawObjects();
	}
}
#endregion

#region Button Click Actions
private void btnAuto_Click(object sender, EventArgs e)
{
	if (btnAuto.Text == "Auto Off")
	{
		if(MessageBox.Show("Are you sure you want to switch to Auto Trading?",
				"Auto Trade",
				MessageBoxButtons.YesNo,
				MessageBoxIcon.Question) == DialogResult.Yes)
		{
			btnAuto.Text = "Auto On";
			btnAuto.Font =boldFont ;
			btnAuto.ForeColor = Color.Gainsboro;
			btnAuto.BackColor=Color.Green;
		}
	}
	else
	{
		btnAuto.Text = "Auto Off";
		btnAuto.Font =regularFont ;
		btnAuto.ForeColor = Color.Gainsboro;
		btnAuto.BackColor=Color.DarkGoldenrod;
	}
	btnAuto.Enabled = true;
}

private void btnLongEntry_Click(object sender, EventArgs e)
{
	if (btnLongEntry.Text == "No Longs")
	{
	btnLongEntry.Text = "Longs On";
	btnLongEntry.Font = boldFont ;
	btnLongEntry.ForeColor = Color.Green;
	btnLongEntry.BackColor=Color.Transparent;
	}
	else
	{
	btnLongEntry.Text = "No Longs";
	btnLongEntry.Font = regularFont ;
	btnLongEntry.ForeColor = Color.Red;
	btnLongEntry.BackColor=Color.Transparent;
	}

	btnLongEntry.Enabled = true;
}

private void btnShortEntry_Click(object sender, EventArgs e)
{
	if (btnShortEntry.Text == "No Shorts")
	{
	btnShortEntry.Text = "Shorts On";
	btnShortEntry.Font = boldFont ;
	btnShortEntry.ForeColor = Color.Green;
	btnShortEntry.BackColor=Color.Transparent;
	}
	else
	{
	btnShortEntry.Text = "No Shorts";
	btnShortEntry.Font = regularFont ;
	btnShortEntry.ForeColor = Color.Red;
	btnShortEntry.BackColor=Color.Transparent;
	}

	btnLongEntry.Enabled = true;
}

private void btnBasicTrend_Click(object sender, EventArgs e)
{
	if (btnBasicTrend.Text == "BasicTrend Off")
	{
	btnBasicTrend.Text = "BasicTrend On";
	btnBasicTrend.Font = boldFont ;
	btnBasicTrend.ForeColor = Color.Green;
	btnBasicTrend.BackColor=Color.Transparent;
	}
	else
	{
	btnBasicTrend.Text = "BasicTrend Off";
	btnBasicTrend.Font = regularFont ;
	btnBasicTrend.ForeColor = Color.Red;
	btnBasicTrend.BackColor=Color.Transparent;
	}

	btnBasicTrend.Enabled = true;
}
private void btnReEntry_Click(object sender, EventArgs e)
{
	if (btnReEntry.Text == "ReEntry Off")
	{
	btnReEntry.Text = "ReEntry On";
	btnReEntry.Font = boldFont ;
	btnReEntry.ForeColor = Color.Green;
	btnReEntry.BackColor=Color.Transparent;
	}
	else
	{
	btnReEntry.Text = "ReEntry Off";
	btnReEntry.Font = regularFont ;
	btnReEntry.ForeColor = Color.Red;
	btnReEntry.BackColor=Color.Transparent;
	}

	btnReEntry.Enabled = true;
}
private void btn3BR_Click(object sender, EventArgs e)
{
	if (btn3BR.Text == "3BR Off")
	{
	btn3BR.Text = "3BR On";
	btn3BR.Font = boldFont ;
	btn3BR.ForeColor = Color.Green;
	btn3BR.BackColor=Color.Transparent;
	}
	else
	{
	btn3BR.Text = "3BR Off";
	btn3BR.Font = regularFont ;
	btn3BR.ForeColor = Color.Red;
	btn3BR.BackColor=Color.Transparent;
	}

	btn3BR.Enabled = true;
}
private void btnBoxBreak_Click(object sender, EventArgs e)
{
	if (btnBoxBreak.Text == "BoxBreak Off")
	{
	btnBoxBreak.Text = "BoxBreak On";
	btnBoxBreak.Font = boldFont ;
	btnBoxBreak.ForeColor = Color.Green;
	btnBoxBreak.BackColor=Color.Transparent;
	}
	else
	{
	btnBoxBreak.Text = "BoxBreak Off";
	btnBoxBreak.Font = regularFont ;
	btnBoxBreak.ForeColor = Color.Red;
	btnBoxBreak.BackColor=Color.Transparent;
	}

	btnBoxBreak.Enabled = true;
}

private void btnOrderType1_Click(object sender, EventArgs e)
{
	if (btnOrderType1.Text == "Stop Market")
	{
		btnOrderType1.Text = "Market";
		btnOrderType1.Font = boldFont ;
		btnOrderType1.ForeColor = Color.Red;
		btnOrderType1.BackColor=Color.Transparent;
	}
	else
	{
	btnOrderType1.Text = "Stop Market";
	btnOrderType1.Font = regularFont ;
	btnOrderType1.ForeColor = Color.Green;
	btnOrderType1.BackColor=Color.Transparent;
	}

	btnOrderType1.Enabled = true;
}
#endregion

#region Manual Button Entries
private void btnManualLongEntry_Click(object sender, EventArgs e)
{
	if (btnOrderType1.Text == "Stop Market")
	{
		{string[] status = GetAtmStrategyEntryOrderStatus(orderId);
				if (status.GetLength(0) > 0)
					{
						if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
							orderId = string.Empty;
					}
				else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
							atmStrategyId = string.Empty;
		}
		int barsize = BarsPeriod.Value;
		double upper = Low[-1]+(barsize)*TickSize;	
		atmStrategyId = GetAtmStrategyUniqueId();
		orderId = GetAtmStrategyUniqueId();
		AtmStrategyCreate(OrderAction.Buy,OrderType.Stop,0,upper+1*TickSize,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
	}
	if (btnOrderType1.Text == "Market")
	{
		{
				string[] status = GetAtmStrategyEntryOrderStatus(orderId);
				if (status.GetLength(0) > 0)
					{
						if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
							orderId = string.Empty;
					}
				
				else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
							atmStrategyId = string.Empty;    
			}
		atmStrategyId = GetAtmStrategyUniqueId();
		orderId = GetAtmStrategyUniqueId();
		AtmStrategyCreate(OrderAction.Buy,OrderType.Market,0,0,TimeInForce.Day,orderId,manualTemplate,atmStrategyId);
	}
	btnManualLongEntry.Enabled = true;
}

private void btnManualShortEntry_Click(object sender, EventArgs e)
{
	if (btnOrderType1.Text == "Stop Market")
	{
		if (orderId.Length > 0)
			{
				string[] status = GetAtmStrategyEntryOrderStatus(orderId);
				if (status.GetLength(0) > 0)
					{
						if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
							orderId = string.Empty;
					}
				
				else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
							atmStrategyId = string.Empty;    
			}
		atmStrategyId = GetAtmStrategyUniqueId();
		orderId = GetAtmStrategyUniqueId();	
		int barsize = BarsPeriod.Value;
		double lower = High[-1]-(barsize)*TickSize;
		AtmStrategyCreate(OrderAction.Sell,OrderType.Stop,0,lower-1*TickSize,TimeInForce.Gtc,orderId,manualTemplate,atmStrategyId);
	}
	if (btnOrderType1.Text == "Market")
	{
		if (orderId.Length > 0)
			{
				string[] status = GetAtmStrategyEntryOrderStatus(orderId);
				if (status.GetLength(0) > 0)
					{
						if (status[2] == "Filled" || status[2] == "Cancelled" || status[2] == "Rejected")
							orderId = string.Empty;
					}
				
				else if (atmStrategyId.Length > 0 && GetAtmStrategyMarketPosition(atmStrategyId) == Cbi.MarketPosition.Flat)
							atmStrategyId = string.Empty;    
			}
		atmStrategyId = GetAtmStrategyUniqueId();
		orderId = GetAtmStrategyUniqueId();
		AtmStrategyCreate(OrderAction.Sell,OrderType.Market,0,0,TimeInForce.Day,orderId,manualTemplate,atmStrategyId);
	}
	btnManualShortEntry.Enabled = true;
}

 private void btnClose_Click(object sender, EventArgs e)
{
	ExitLong(quantity*2);
	ExitShort();
	AtmStrategyClose(atmStrategyId);
	atmStrategyId = string.Empty;
	orderId = string.Empty;
	btnClose.Enabled = true;
}
#endregion
		
#region Properties

		#region A.ATM Templates
		
		[Description("ATM Strategy Name for Auto and Manual Entries - must be written exactly as the ATM Strategy is named.")]
		[GridCategory("A.ATM Strategy")]
		public string ATMTemplate
		{
			get {	return manualTemplate;	}
			set	{	manualTemplate = value;	}
		}
		[Description("ATM Strategy Name for Auto Box Trade Entries Only - must be written exactly as the ATM Strategy is named.")]
		[GridCategory("A.ATM Strategy")]
		public string BoxBreakTemplate
		{
			get {	return boxTemplate;	}
			set	{	boxTemplate = value;	}
		}
		#endregion
		
		#region B.Time
        [Description("Start Hour")]
        [GridCategory("B.Time")]
        public int StartHour
        {
            get { return hourStart; }
            set { hourStart = Math.Max(0, value); }
        }
        [Description("Start Minute")]
        [GridCategory("B.Time")]
        public int StartMinute
        {
            get { return minuteStart; }
            set { minuteStart = Math.Max(0, value); }
        }
        [Description("End Hour")]
        [GridCategory("B.Time")]
        public int StopHour
        {
            get { return hourEnd; }
            set { hourEnd = Math.Max(0, value); }
        }
        [Description("End Minute")]
        [GridCategory("B.Time")]
        public int StopMinute
        {
            get { return minuteEnd; }
            set { minuteEnd = Math.Max(0, value); }
        }
		#endregion
		
		#region C.Daily Profit & Loss Limit
		//P & L, Targets & Stops Publics
        [Description("Max Daily Profit Limit")]
        [GridCategory("C.P&L Limits")]
        public int MaxWin
        {
            get { return maxWin; }
            set { maxWin = Math.Max(1, value); }
        }
        [Description("Max Daily Loss Limit")]
        [GridCategory("C.P&L Limits")]
        public int MaxLoss
        {
            get { return maxLoss; }
            set { maxLoss = Math.Max(0, value); }
        }
		#endregion
		
		#region D.Directional Filter
		[Description("Long Trades On")]
        [GridCategory("D.Directional Filter")]
        public bool LongTradesOn
        {
            get { return longs; }
            set { longs = value; }
        }
		[Description("Short Trades On")]
        [GridCategory("D.Directional Filter")]
        public bool ShortTradesOn
        {
            get { return shorts; }
            set { shorts = value; }
        }
		#endregion
		
		#region E.Trade Types
		
		[Description("Lower high / higher low pullback pattern - must have Re-Entry trade enable for this trade to fire.")]
        [GridCategory("E.Trade Types")]
        public bool PullbackPattern
        {
            get { return pullbackPattern; }
            set { pullbackPattern = value; }
        }
		
		#endregion
		
		#region F.Trigger ATR Settings
		[Description("ATR Period")]
        [GridCategory("F.ATR Trigger Settings")]
        public int ATRPeriod
        {
            get { return aTRPeriod; }
            set { aTRPeriod = Math.Max(1, value); }
        }
		
		[Description("ATR Multiplier")]
        [GridCategory("F.ATR Trigger Settings")]
        public double ATRMultiplier
        {
            get { return aTRMultiplier; }
            set { aTRMultiplier = Math.Max(1, value); }
        }
		
		[Description("ATR Smoothing")]
        [GridCategory("F.ATR Trigger Settings")]
        public int ATRSmoothing
        {
            get { return aTRSmoothing; }
            set { aTRSmoothing = Math.Max(1, value); }
        }
		#endregion
		
		#region G.Filter ATR Settings
		[Description("ATR Period")]
        [GridCategory("G.ATR Filter Settings")]
        public int FilterATRPeriod
        {
            get { return bigATRPeriod; }
            set { bigATRPeriod = Math.Max(1, value); }
        }
		
		[Description("ATR Multiplier")]
        [GridCategory("G.ATR Filter Settings")]
        public double FilterATRMultiplier
        {
            get { return bigATRMultiplier; }
            set { bigATRMultiplier = Math.Max(1, value); }
        }
		
		[Description("ATR Smoothing")]
        [GridCategory("G.ATR Filter Settings")]
        public int FilterATRSmoothing
        {
            get { return bigATRSmoothing; }
            set { bigATRSmoothing = Math.Max(1, value); }
        }
		
		[Description("Use HTF ATR Filter")]
        [GridCategory("G.ATR Filter Settings")]
        public bool UseATRFilter
        {
            get { return bigATRFilter; }
            set { bigATRFilter = value; }
        }
		#endregion
		
		#region H.CCI Settings
        [Description("CCI Period")]
        [GridCategory("H.CCI Settings")]
        public int CCIFilterPeriod
        {
            get { return cCIFilterPeriod; }
            set { cCIFilterPeriod = Math.Max(1, value); }
        }
        [Description("CCI Filter Level")]
        [GridCategory("H.CCI Settings")]
        public int CCIFilterLevel
        {
            get { return cCIFilterLevel; }
            set { cCIFilterLevel = Math.Max(0, value); }
        }
		[Description("Use CCI Filter")]
        [GridCategory("H.CCI Settings")]
        public bool UseCCIFilter
        {
            get { return useCCIFilter; }
            set { useCCIFilter = value; }
        }
		[Description("Use CCI Overbought or Oversol to exit trade")]
        [GridCategory("H.CCI Settings")]
        public bool UseCCIOBOS
        {
            get { return useCCIOBOS; }
            set { useCCIOBOS = value; }
        }
		#endregion
	
		#region I.Keltner Settings
		[Description("Keltner Period")]
        [GridCategory("I.Keltner Settings")]
        public int KeltnerPeriod
        {
            get { return keltnerPeriod; }
            set { keltnerPeriod = Math.Max(1, value); }
        }
		[Description("Keltner Exit")]
        [GridCategory("I.Keltner Settings")]
        public bool UseKeltnerExit
        {
            get { return useKeltnerExit; }
            set { useKeltnerExit = value; }
        }
		
		[Description("Distance to Midband Filter")]
        [GridCategory("I.Keltner Settings")]
        public int DistanceToMidband
        {
            get { return distanceToMA; }
            set { distanceToMA = Math.Max(1, value); }
        }
		
		[Description("Use Midband Filter")]
        [GridCategory("I.Keltner Settings")]
        public bool UseMidbandFilter
        {
            get { return useMAFilter; }
            set { useMAFilter = value; }
        }
        [Description("Keltner Multiplier")]
        [GridCategory("I.Keltner Settings")]
        public double KeltnerMultiplier
        {
            get { return keltnerMultiplier; }
            set { keltnerMultiplier = Math.Max(0, value); }
        }
		
		[Description("Take Trades only on correct side of Midband")]
        [GridCategory("I.Keltner Settings")]
        public bool MidbandBias
        {
            get { return midbandBias; }
            set { midbandBias = value; }
        }
		#endregion
		
		#region J.Moving Average Settings
		[Description("Using moving average slope as a bias for trades")]
        [GridCategory("J.Moving Average Settings")]
        public bool UseMASlope
        {
            get { return useMARisingFalling; }
            set { useMARisingFalling = value; }
        }
		[Description("Moving average period")]
        [GridCategory("J.Moving Average Settings")]
        public int MovingAveragePeriod
        {
            get { return mAPeriod; }
            set { mAPeriod = Math.Max(1, value); }
        }
		#endregion
		
		#region K.3BR Trade Settings
		[Description("Pullback must be a certain distance from the moving average (in ticks) to look for 3BR trades")]
        [GridCategory("K.3BR")]
        public bool UseDistanceToMAFilter
        {
            get { return useDistanceToMAFilter; }
            set { useDistanceToMAFilter = value; }
        }
		[Description("3BR must be within x ticks of the moving average to take the 3BR trade")]
        [GridCategory("K.3BR")]
        public int DistanceToMA3BR
        {
            get { return distanceToMA3BR; }
            set { distanceToMA3BR = Math.Max(1, value); }
        }
		[Description("Price must be within x ticks or beyond Keltner outerband for a countertrend reversal trade")]
        [GridCategory("K.3BR")]
        public bool UseKeltnerFilter
        {
            get { return useKeltnerFilter; }
            set { useKeltnerFilter = value; }
        }
		[Description("3BR must be within x ticks from the Keltner outerband to take the 3BR countertrend trade")]
        [GridCategory("K.3BR")]
        public int DistanceToKeltner
        {
            get { return distanceToKeltner; }
            set { distanceToKeltner = Math.Max(1, value); }
        }
		[Description("Target for Second Contract on 3BR Trades")]
        [GridCategory("K.3BR")]
        public int Target2for3BR
        {
            get { return target2BR; }
            set { target2BR = Math.Max(1,value); }
        }
		[Description("Three Bar Reversal as Exit")]
        [GridCategory("K.3BR")]
        public bool ThreeBarReversalExit
        {
            get { return threeBRExit; }
            set { threeBRExit = value; }
        }
		[Description("Using above/below Keltner Midband as a bias for 3BR trades - pullback trades only")]
        [GridCategory("K.3BR")]
        public bool UseMidBandBias
        {
            get { return useMABiasFilter; }
            set { useMABiasFilter = value; }
        }
		#endregion
		
		#region L.Box Trade Settings
		[Description("How many bars after the pattern first appears until the pending stop order will be cancelled.")]
        [GridCategory("L.Box Trade")]
        public int BarsBeforeOrderCancel
        {
            get { return barsCancel; }
            set { barsCancel = Math.Max(1,value); }
        }
		
		#endregion
		
		#region M.Re-Entry Trade Settings
		[Description("Re-entry Trade from ATR requires two closes in direction of trade prior to entry")]
        [GridCategory("M.Re-Entry Trade")]
        public bool Use2BarEntry
        {
            get { return use2BarEntry; }
            set { use2BarEntry = value; }
        }
		[Description("Distance from Trigger ATR for Re-Entry Trade")]
        [GridCategory("M.Re-Entry Trade")]
        public int ReEntryTickDistance
        {
            get { return reEntryTickDistance; }
            set { reEntryTickDistance = Math.Max(0, value); }
        }
		[Description("Use CCI Filter for ReEntry trades")]
        [GridCategory("M.Re-Entry Trade")]
        public bool UseCCIforReEntry
        {
            get { return useCCIReEntry; }
            set { useCCIReEntry = value; }
        }
		#endregion
		

  #endregion
		
	}
}
